/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: ByteArray.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_ByteArray_H
#define OpenSOAP_ByteArray_H

#include <stdlib.h>
#include <sys/types.h>

#include <OpenSOAP/Defines.h>
#include <OpenSOAP/ErrorCode.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPByteArray OpenSOAPByteArray;
    typedef OpenSOAPByteArray    *OpenSOAPByteArrayPtr;

    /* create 0 size OpenSOAPByteArray */
    int
    OPENSOAP_API
    OpenSOAPByteArrayCreate(OpenSOAPByteArrayPtr * /* [out] */ b_ary);

    /* create OpenSOAPByteArray with initial data */
    int
    OPENSOAP_API
    OpenSOAPByteArrayCreateWithData(const unsigned char * /* [in] */ data,
                                    size_t /* [in] */ sz,
                                    OpenSOAPByteArrayPtr * /* [out] */ b_ary);

    /* create OpenSOAPByteArray with initial size */
#define OpenSOAPByteArrayCreateWithSize(sz, b_ary) \
    OpenSOAPByteArrayCreateWithData(0, (sz), (b_ary))

    /* Use OpenSOAPByteArray Resource declare */
    int
    OPENSOAP_API
    OpenSOAPByteArrayAppendRef(OpenSOAPByteArrayPtr /* [in] */ b_ary);

    /* release OpenSOAPByteArray Resource */
    int
    OPENSOAP_API
    OpenSOAPByteArrayRelease(OpenSOAPByteArrayPtr /* [in] */ b_ary);

    /* get size */
    int
    OPENSOAP_API
    OpenSOAPByteArrayGetSize(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                             size_t * /* [out] */ sz);

    /* get first element const pointer */
    int
    OPENSOAP_API
    OpenSOAPByteArrayBeginConst(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                                const unsigned char ** /* [out] */beg);
    
    /* get terminate element const pointer */
    int
    OPENSOAP_API
    OpenSOAPByteArrayEndConst(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                              const unsigned char ** /* [out] */ed);
    
    /* get first element pointer and size */
    int
    OPENSOAP_API
    OpenSOAPByteArrayGetBeginSizeConst(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                                       const unsigned char ** /* [out] */ beg,
                                       size_t * /* [out] */ sz);

    /* get first and terminate element const pointer */
    int
    OPENSOAP_API
    OpenSOAPByteArrayGetBeginEndConst(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                                      const unsigned char ** /* [out] */ beg,
                                      const unsigned char ** /* [out] */ ed);
    
    /* get first element pointer */
    int
    OPENSOAP_API
    OpenSOAPByteArrayBegin(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                           unsigned char ** /* [out] */beg);
    
    /* get terminate element pointer */
    int
    OPENSOAP_API
    OpenSOAPByteArrayEnd(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                         unsigned char ** /* [out] */ed);

    /* get first element pointer and size */
    int
    OPENSOAP_API
    OpenSOAPByteArrayGetBeginSize(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                                  unsigned char ** /* [out] */ beg,
                                  size_t * /* [out] */ sz);

    /* get first and terminate element pointer */
    int
    OPENSOAP_API
    OpenSOAPByteArrayGetBeginEnd(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                                 unsigned char ** /* [out] */ beg,
                                 unsigned char ** /* [out] */ ed);
    
    /* clear OpenSOAPByteArray */
    int
    OPENSOAP_API
    OpenSOAPByteArrayClear(OpenSOAPByteArrayPtr /* [in] */ b_ary);

    /* set data to OpenSOAPByteArray */
    int
    OPENSOAP_API
    OpenSOAPByteArraySetData(OpenSOAPByteArrayPtr /* [in, out] */ b_ary,
                             const unsigned char * /* [in] */ data,
                             size_t /* [in] */ sz);
    
    /* Append Byte data to OpenSOAPByteArray */
    int
    OPENSOAP_API
    OpenSOAPByteArrayAppend(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                            const unsigned char *dat,
                            size_t       dat_len);

    /* resize OpenSOAPByteArray */

    int
    OPENSOAP_API
    OpenSOAPByteArrayResize(OpenSOAPByteArrayPtr /* [in, out] */ b_ary,
                            size_t       /* [in] */ new_size);

    /* duplicate OpenSOAPByteArray */
    int
    OPENSOAP_API
    OpenSOAPByteArrayDuplicate(OpenSOAPByteArrayPtr /* [in] */ b_ary,
                               OpenSOAPByteArrayPtr * /* [out] */ dup_b_ary);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_ByteArray_H */
