/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Security.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_Security_H
#define OpenSOAP_Security_H

#include <stdio.h>
#include <OpenSOAP/Envelope.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Hash Algorithms */
#define OPENSOAP_HA_MD2    (1)  /* MD2 */
#define OPENSOAP_HA_MD4    (2)  /* MD4 */
#define OPENSOAP_HA_MD5    (3)  /* MD5 */
#define OPENSOAP_HA_MDC2   (4)  /* MDC2 */
#define OPENSOAP_HA_RIPEMD (5)  /* RIPEMD */
#define OPENSOAP_HA_SHA    (6)  /* SHA */

#define OPENSOAP_CA_OWNER_LEN  (1024) /* Max Owner Length in CA Database */
#define OPENSOAP_CERT_DATE_LEN   (14) /* Date Length In Certificate */

    /* Certificate */
    typedef struct tagOpenSOAPSecCert OpenSOAPSecCert;
    typedef OpenSOAPSecCert* OpenSOAPSecCertPtr;
    /* CA */
    typedef struct tagOpenSOAPCARec OpenSOAPCARec;
    typedef OpenSOAPCARec* OpenSOAPCARecPtr;

    /* Generate RSA Keys To Stream */
    int
    OPENSOAP_API
    OpenSOAPSecGenerateRSAKeys
    (const unsigned char* szSeedPhrase, /* (i)  Seed phrase */
     FILE*          fpPrivKey,    /* (i)  RSA Private Key File stream */
     FILE*          fpPubKey);    /* (i)  RSA Public Key File stream */

    /* Generate RSA Keys To File */
    int
    OPENSOAP_API
    OpenSOAPSecGenerateRSAKeysToFile
    (const unsigned char* szSeedPhrase,      /* (i)  Seed phrase */
     const char*          szPrivKeyFileName, /* (i)  RSA Private Key File Name */
     const char*          szPubKeyFileName); /* (i)  RSA Public Key File Name */

    /* Encrypt Envelope Using Public Key File Stream */
    int
    OPENSOAP_API
    OpenSOAPSecEncWithStream
    (OpenSOAPEnvelopePtr env,       /* (i)  Envelope */
     FILE*               fpPubKey); /* (i)  Public Key File stream */
    /* Encrypt Envelope Using Public Key File */
    int
    OPENSOAP_API
    OpenSOAPSecEncWithFile
    (OpenSOAPEnvelopePtr env,         /* (i)  Envelope */
     const char*         szPubKName); /* (i)  Public Key File Name */

    /* Decrypt Envelope Using Public Key File Stream */
    int
    OPENSOAP_API
    OpenSOAPSecDecWithStream
    (OpenSOAPEnvelopePtr env,        /* (i)  Envelope */
     FILE*               fpPrivKey); /* (i)  Private Key File stream */
    /* Encrypt Envelope Using Public Key File */
    int
    OPENSOAP_API
    OpenSOAPSecDecWithFile
    (OpenSOAPEnvelopePtr env,          /* (i)  Envelope */
     const char*         szPrivKName); /* (i)  Private Key File Name */

    /* Add Signature to SOAP Message using PrivateKey File Stream */
    int
    OPENSOAP_API
    OpenSOAPSecAddSignWithStream
    (OpenSOAPEnvelopePtr env,        /* (io) Envelope */
     int                 iType,      /* (i)  Hash Type(OPENSOAP_HA_*) */
     FILE*               fpPrivKey,  /* (i)  Private Key File stream */
     OpenSOAPStringPtr   name);      /* (i)  Name of Sign */

	/* Add Signature to SOAP Message using PrivateKey File */
    int
    OPENSOAP_API
    OpenSOAPSecAddSignWithFile
    (OpenSOAPEnvelopePtr env,          /* (io) Envelope */
     int                 iType,        /* (i)  Hash Type(OPENSOAP_HA_*) */
     const char*         szPrivKName,  /* (i)  Private Key File Name */
     OpenSOAPStringPtr   name);        /* (i)  Name of Sign */

    /* Count Signature */
    int
    OPENSOAP_API
    OpenSOAPSecCntSign
    (const OpenSOAPEnvelopePtr env,    /* (i)  Envelope */
     int*                      pnSig); /* (o)  Signature Count */

    /* Get "SignedBy" List of Signature */
    int
    OPENSOAP_API
    OpenSOAPSecGetSignedByList
    (OpenSOAPEnvelopePtr env,          /* (i)  Envelope */
     int                 nCntMax,      /* (i)  Maximum Count */
     OpenSOAPStringPtr   list[],       /* (io) SignedBy List  */
     int*                pnCntPacked); /* (o)  Packed Count */

    /* Verify Signature using PublicKey File Stream */
    int
    OPENSOAP_API
    OpenSOAPSecVerifySignWithStream
    (OpenSOAPEnvelopePtr env,       /* (io) Envelope */
     FILE*               fpPubKey); /* (i)  Public Key File stream */
    /* Verify Signature using PublicKey File */
    int
    OPENSOAP_API
    OpenSOAPSecVerifySignWithFile
    (OpenSOAPEnvelopePtr env,         /* (io) Envelope */
     const char*         szPubKName); /* (i)  Public Key File Name */

    /* Create Certificate from CA Record(stream) */
    int
    OPENSOAP_API
    OpenSOAPSecCertCreateWithStream
    (const char*            szPublish, /* (i)  Publisher's Name */
     FILE*                  fpPrivKey, /* (i)  Private Key(Publisher) */
     int                    iHashType, /* (i)  Hash Type(OPENSOAP_HA_*) */
     const OpenSOAPCARecPtr pRec,      /* (i)  CA Record */
     FILE*                  fpCert);   /* (o)  Certificate File */
	/* Create Certificate from CA Record(file) */
    int
    OPENSOAP_API
    OpenSOAPSecCertCreateWithFile
    (const char*         szPublish,     /* (i)  Publisher's Name */
     const char*         szPrivKeyFile, /* (i)  PrivKey File Name (Publisher) */
     int                 iHashType,     /* (i)  Hash Type(OPENSOAP_HA_*) */
     const OpenSOAPCARecPtr pRec,          /* (i)  CA Record */
     const char*         szCertName);   /* (i)  Certificate File Name */
    /* Load Certificate From Memory */
    int
    OPENSOAP_API
    OpenSOAPSecCertLoadFromMem
    (size_t               sizArea,  /* (i)  Size of Input Area */
     const unsigned char* pucArea,  /* (i)  Input Area */
     OpenSOAPSecCertPtr*  ppCert);  /* (o)  Certificate */
    /* Load Certificate From File */
    int
    OPENSOAP_API
    OpenSOAPSecCertLoad
    (const char*          szName,   /* (i)  File Name */
     OpenSOAPSecCertPtr*  ppCert);  /* (o)  Certificate */
    /* Free Cretificate */
    int
    OPENSOAP_API
    OpenSOAPSecCertFree
    (OpenSOAPSecCertPtr pCert);
    /* Verify Certificate (Stream) */
    int
    OPENSOAP_API
    OpenSOAPSecCertVerifyWithStream
    (FILE* fpCert,                  /* (i)  Certificate File Stream */
     FILE* fpPubKey);               /* (i)  Public Key File Stream */
    /* Verify Certificate (File) */
    int
    OPENSOAP_API
    OpenSOAPSecCertVerifyWithFile
    (const char* szCertName,    /* (i)  Certificate File Name */
     const char* szPubKeyName); /* (i)  Public Key File Name */
    /* Get Name Of Publisher */
    int
    OPENSOAP_API
    OpenSOAPSecCertGetPublisherName
    (OpenSOAPSecCertPtr pCert,      /* (i)   */
     char**             pszName);   /* (o)  Name (Internal Area) */
    /* Get Serial Number */
    int
    OPENSOAP_API
    OpenSOAPSecCertGetSerialNo
    (OpenSOAPSecCertPtr pCert,      /* (i)   */
     unsigned long*     pulSerial); /* (o)  Serial Number */
    /* Get Name of Owner */
    int
    OPENSOAP_API
    OpenSOAPSecCertGetOwnerName
    (OpenSOAPSecCertPtr pCert,      /* (i)   */
     char**             pszName);   /* (o)  Name (Internal Area) */
    /* Get End Date */
    int
    OPENSOAP_API
    OpenSOAPSecCertGetEndDate
    (OpenSOAPSecCertPtr pCert,      /* (i)   */
     char**             pszDate);   /* (o)  Date(Internal Area) */
    /* Get Public Key */
    int
    OPENSOAP_API
    OpenSOAPSecCertGetPubKey
    (OpenSOAPSecCertPtr pCert,       /* (i)   */
     const char*        szSaveName); /* (i)  Public Key File Name */

    /* Decode Key File */
    int
    OPENSOAP_API
    OpenSOAPSecDecodeKeyFile
    (FILE*           fp,          /* (i)  File Stream */
     unsigned long*  pulLenOut,   /* (o)  Length of output data */
     unsigned char** ppucDecode); /* (o)  Decoded String(call free() after use) */

    /* Browse CA Record */
    int
    OPENSOAP_API
    OpenSOAPSecCABrowseRec
    (const OpenSOAPCARecPtr pRec,   /* (i)  Record */
     FILE*                  fpOut); /* (i)  Output Stream */
    /* Browse CA Database */
    int
    OPENSOAP_API
    OpenSOAPSecCABrowse
    (FILE* fpOut);                  /* (i)  Output Stream */
    /* Regist Key To CA Databse */ 
    int
    OPENSOAP_API
    OpenSOAPSecCARegist
    (const char*    szNameOwner,    /* (i)  Owner's Name */
     const char*    szTermDate,     /* (i)  Terminate Date */
                                    /*      "YYYYMMDDHHMMSS" */
     size_t         sizPubkey,      /* (i)  Size of public key */
     const unsigned char* szPubKey, /* (i)  Public Key */
     unsigned long* pulSerialNo);   /* (o)  Serial Number */
    /* Invalidate CA Record */
    int
    OPENSOAP_API
    OpenSOAPSecCAInvalidate
    (const char*     szNameOwner,   /* (i)  Owner's Name */
     unsigned long   ulSerial);     /* (i)  Serial Number */
    /* Search Records */
    int
    OPENSOAP_API
    OpenSOAPSecCASearchRecords
    (const char* szNameOwner, /* (i)  Owner's Name */
     int*        pnRec,       /* (o)  Record Count */
     long**      pplIdxs);    /* (o)  Index Numbers (call free() after use) */
    /* Search Record(Longest Period) */
    int
    OPENSOAP_API
    OpenSOAPSecCASearchOneRecord
    (const char*         szNameOwner, /* (i)  Owner's Name */
     OpenSOAPCARecPtr*   ppRec);      /* (o)  Record */
    /* Get Record */
    int
    OPENSOAP_API
    OpenSOAPSecCAGetRecord
    (long              lIdx,    /* (i)  Index */
     OpenSOAPCARecPtr* ppRec);  /* (o)  Record */
    /* Free Record */
    int
    OPENSOAP_API
    OpenSOAPSecCAFreeRecord
    (OpenSOAPCARecPtr pRec);  /* (i)  Record */
    /* Remove CA Record */
    int
    OPENSOAP_API
    OpenSOAPSecCARemoveRecord
    (unsigned long ulSerial);   /* (i)  Serial Number */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_Security_H */
