/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: XMLElm.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_XMLElm_H
#define OpenSOAP_XMLElm_H

#include <OpenSOAP/XMLAttr.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPXMLElm OpenSOAPXMLElm;
    typedef OpenSOAPXMLElm *OpenSOAPXMLElmPtr;

    int
    OPENSOAP_API
    OpenSOAPXMLElmCreate(/* [out] */ OpenSOAPXMLElmPtr *elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetNamespaceMB(/* [out] */ OpenSOAPXMLElmPtr elm,
                                 /* [in]  */ const char *ns_uri,
                                 /* [in]  */ const char *ns_prefix);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetNamespaceWC(/* [out] */ OpenSOAPXMLElmPtr elm,
                                 /* [in]  */ const wchar_t *ns_uri,
                                 /* [in]  */ const wchar_t *ns_prefix);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSearchNamespaceMB(/* [in]  */ OpenSOAPXMLElmPtr elm,
                                    /* [in]  */ const char *ns_uri,
                                    /* [in]  */ const char *ns_prefix,
                                    /* [out] */ OpenSOAPXMLNamespacePtr *ns);
    
    int
    OPENSOAP_API
    OpenSOAPXMLElmSearchNamespaceWC(/* [in]  */ OpenSOAPXMLElmPtr elm,
                                    /* [in]  */ const wchar_t *ns_uri,
                                    /* [in]  */ const wchar_t *ns_prefix,
                                    /* [out] */ OpenSOAPXMLNamespacePtr *ns);
    
    int
    OPENSOAP_API
    OpenSOAPXMLElmGetNamespace(/* [in]  */ OpenSOAPXMLElmPtr elm,
                               /* [out] */ OpenSOAPXMLNamespacePtr *ns);

    int
    OPENSOAP_API
    OpenSOAPXMLElmDefineNamespaceMB(/* [out] */ OpenSOAPXMLElmPtr elm,
                                    /* [in]  */ const char *ns_uri,
                                    /* [in]  */ const char *ns_prefix,
                                    /* [out] */ OpenSOAPXMLNamespacePtr *ns);

    int
    OPENSOAP_API
    OpenSOAPXMLElmDefineNamespaceWC(/* [out] */ OpenSOAPXMLElmPtr elm,
                                    /* [in]  */ const wchar_t *ns_uri,
                                    /* [in]  */ const wchar_t *ns_prefix,
                                    /* [out] */ OpenSOAPXMLNamespacePtr *ns);

    int
    OPENSOAP_API
    OpenSOAPXMLElmAddAttributeMB(/* [out] */ OpenSOAPXMLElmPtr elm,
                                 /* [in]  */ const char *attr_name,
                                 /* [in]  */ const char *attr_type,
                                 /* [in]  */ void *attr_value,
                                 /* [out] */ OpenSOAPXMLAttrPtr *attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmAddAttributeWC(/* [out] */ OpenSOAPXMLElmPtr elm,
                                 /* [in]  */ const wchar_t *attr_name,
                                 /* [in]  */ const wchar_t *attr_type,
                                 /* [in]  */ void *attr_value,
                                 /* [out] */ OpenSOAPXMLAttrPtr *attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetAttributeMB(/* [in]  */ OpenSOAPXMLElmPtr elm,
                                 /* [in]  */ const char *attr_name,
                                 /* [out] */ OpenSOAPXMLAttrPtr *attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetAttributeWC(/* [in]  */ OpenSOAPXMLElmPtr elm,
                                 /* [in]  */ const wchar_t *attr_name,
                                 /* [out] */ OpenSOAPXMLAttrPtr *attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetNextChild(/* [in]  */ OpenSOAPXMLElmPtr elm,
                               /* [in, out] */ OpenSOAPXMLElmPtr *cld_elm);

	int
	OPENSOAP_API
	OpenSOAPXMLElmAddChildString(/* [in]  */ OpenSOAPXMLElmPtr elm,
								 /* [in]  */ OpenSOAPStringPtr childName,
								 /* [out] */ OpenSOAPXMLElmPtr *childElm);

	int
    OPENSOAP_API
    OpenSOAPXMLElmAddChildMB(/* [in]  */ OpenSOAPXMLElmPtr elm,
                             /* [in]  */ const char *cld_name,
                             /* [out] */ OpenSOAPXMLElmPtr *cld_elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmAddChildWC(/* [in]  */ OpenSOAPXMLElmPtr elm,
                             /* [in]  */ const wchar_t *cld_name,
                             /* [out] */ OpenSOAPXMLElmPtr *cld_elm);

	int
	OPENSOAP_API
	OpenSOAPXMLElmGetChildString(/* [in]  */ OpenSOAPXMLElmPtr elm,
								 /* [in]  */ OpenSOAPStringPtr chldName,
								 /* [out] */ OpenSOAPXMLElmPtr *chldElm);
	
    int
    OPENSOAP_API
    OpenSOAPXMLElmGetChildMB(/* [in]  */ OpenSOAPXMLElmPtr elm,
                             /* [in]  */ const char *cld_name,
                             /* [out] */ OpenSOAPXMLElmPtr *cld_elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetChildWC(/* [in]  */ OpenSOAPXMLElmPtr elm,
	                         /* [in]  */ const wchar_t *cld_name,
                             /* [out] */ OpenSOAPXMLElmPtr *cld_elm);


    int
    OPENSOAP_API
    OpenSOAPXMLElmGetValueMB(/* [in]  */ OpenSOAPXMLElmPtr elm,
                             /* [in]  */ const char *type_name,
                             /* [out] */ void *value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetValueWC(/* [in]  */ OpenSOAPXMLElmPtr elm,
                             /* [in]  */ const wchar_t *type_name,
                             /* [out] */ void *value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetValueMB(/* [out] */ OpenSOAPXMLElmPtr elm,
                             /* [in]  */ const char *type_name,
                             /* [in]  */ void *value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetValueWC(/* [out] */ OpenSOAPXMLElmPtr elm,
                             /* [in]  */ const wchar_t *type_name,
                             /* [in]  */ void *value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetCharEncodingString(/* [in]  */ OpenSOAPXMLElmPtr elm,
                                        /* [in]  */ const char *chEnc,
                                        /* [out] */ OpenSOAPByteArrayPtr bAry);

	/* get XML Element's name as OpenSOAPString */
    int
    OPENSOAP_API
    OpenSOAPXMLElmGetNameString(/* [in]  */ OpenSOAPXMLElmPtr elm,
								/* [out] */ OpenSOAPStringPtr *name);
	
	int
	OPENSOAP_API
	OpenSOAPXMLElmGetNextAttr(/* [in] */ OpenSOAPXMLElmPtr elm,
							  /* [in, out] */ OpenSOAPXMLAttrPtr *attr_elm);

	/* */
    int
    OPENSOAP_API
    OpenSOAPXMLElmGetChildValueMB(/* [in]  */ OpenSOAPXMLElmPtr elm,
								  /* [in]  */ const char *pName,
								  /* [in]  */ const char *typeName,
								  /* [out] */ void *value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetChildValueWC(/* [in]  */ OpenSOAPXMLElmPtr elm,
								  /* [in]  */ const wchar_t *pName,
								  /* [in]  */ const wchar_t *typeName,
								  /* [out] */ void *value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetChildValueMB(/* [in]  */ OpenSOAPXMLElmPtr elm,
								  /* [in]  */ const char *pName,
								  /* [in]  */ const char *typeName,
								  /* [out] */ void *value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetChildValueWC(/* [in]  */ OpenSOAPXMLElmPtr elm,
								  /* [in]  */ const wchar_t *pName,
								  /* [in]  */ const wchar_t *typeName,
								  /* [out] */ void *value);

	/* */
	int
    OPENSOAP_API
	OpenSOAPXMLElmSetValueAsString(/* [out] */ OpenSOAPXMLElmPtr elm,
								   /* [in]  */ OpenSOAPStringPtr value,
								   /* [in]  */ int isDup);

	int
    OPENSOAP_API
	OpenSOAPXMLElmSetValueAsStringMB(/* [out] */ OpenSOAPXMLElmPtr elm,
									 /* [in]  */ const char *value);
	
	int
    OPENSOAP_API
	OpenSOAPXMLElmSetValueAsStringWC(/* [out] */ OpenSOAPXMLElmPtr elm,
									 /* [in]  */ const wchar_t *value);
	/* */
	int
	OPENSOAP_API
	OpenSOAPXMLElmSetChildValueAsStringMB(/* [out] */ OpenSOAPXMLElmPtr elm,
										  /* [in]  */ const char *childName,
										  /* [in]  */ const char *value);

	int
	OPENSOAP_API
	OpenSOAPXMLElmSetChildValueAsStringWC(/* [out] */ OpenSOAPXMLElmPtr elm,
										  /* [in]  */ const wchar_t *childName,
										  /* [in]  */ const wchar_t *value);

	/* */
	int
	OPENSOAP_API
	OpenSOAPXMLElmSetChildValueAsString(/* [out] */ OpenSOAPXMLElmPtr elm,
										/* [in]  */
										OpenSOAPStringPtr childName,
										/* [in]  */ int isValueDup,
										/* [in, out] */
										OpenSOAPStringPtr value);
	
	/* */
	int
	OPENSOAP_API
	OpenSOAPXMLElmRemoveChildString(/* [in]  */ OpenSOAPXMLElmPtr elm,
									/* [in]  */ OpenSOAPStringPtr childName,
									/* [in]  */ int isValueRelease,
									/* [out] */ OpenSOAPStringPtr *childValue);	
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_XMLElm_H */
