/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: MsgAttrHandler.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <string>
#include <iostream>
#include <string>

#include <stdexcept>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

/* libxml2 */
#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

#include "MapUtil.h"
#include "ServerCommon.h"
#include "SSMLAttrMgrProtocol.h"
#include "XmlQuery.h"
#include "MsgAttrHandler.h"

using namespace OpenSOAP;
using namespace std;

// for LOG
static const std::string CLASS_SIG = "MsgAttrHandler";

MsgAttrHandler::MsgAttrHandler()
  : docPtr_(NULL)
{
  
}

MsgAttrHandler::MsgAttrHandler(const std::string& xmlMsg)
  : docPtr_(NULL)
{
  if (0 != setMsg(xmlMsg)) {
    throw std::exception();
  }
}

MsgAttrHandler::~MsgAttrHandler()
{
  if (docPtr_) {
    xmlFreeDoc(docPtr_);
  }
}

int MsgAttrHandler::setMsg(const std::string& xmlMsg)
{
  /* COMPAT: Do not generate nodes for formatting spaces */

  LIBXML_TEST_VERSION    xmlKeepBlanksDefault(0);

  char* xmlMsgChar = new char[xmlMsg.length()+1];
  memcpy (xmlMsgChar, xmlMsg.c_str(), xmlMsg.length());
  xmlMsgChar[xmlMsg.length()] = 0x00;

#ifdef DEBUG
  cerr << "XXXXXXXXX MsgAttrHandler::xmlParseMem XXXXXXXXXXXX" << endl;
#endif

  docPtr_ = xmlParseMemory(xmlMsgChar, xmlMsg.length());
  //docPtr_ = xmlParseMemory((char*)xmlMsg.c_str(), xmlMsg.length());
#ifdef DEBUG
  cerr << "XXXXXXXXX MsgAttrHandler::xmlParseMem done. XXXXXXXXXXXX" << endl;
#endif

  if (NULL == docPtr_) {
    cerr << TAG_ERR << CLASS_SIG << "::setMsg: xmlParseMemory returned NULL"
	 << endl;
	delete[] xmlMsgChar;
	return -1;
  }

  /* check document (root node) */
  xmlNodePtr node1 = xmlDocGetRootElement(docPtr_);
  if (node1 == NULL) {
    cerr << TAG_ERR << CLASS_SIG << "::setMsg: empty document"
	 << endl;
    xmlFreeDoc(docPtr_);
	docPtr_ = NULL;
	delete[] xmlMsgChar;
    return -1;
  }

	delete[] xmlMsgChar;

	return 0;
}

int MsgAttrHandler::queryXml(const std::string& query, 
			      std::vector<std::string>& values)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::queryXml() arg=[" << query << "]" << endl;
#endif /* DEBUG */

  XmlQuery* xmlQuery = new XmlQuery(query);
  
  xmlNodePtr node = xmlDocGetRootElement(docPtr_);
  
  int ret = xmlQuery->getValue(docPtr_, node, values);

#ifdef DEBUG
  cerr << "xmlQuery ret=(" << ret << ")" << endl;
#endif /* DEBUG */

  return ret;
}

