/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: QueueManager.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if defined(WIN32)
#else
//#include <sys/types.h>
//#include <sys/socket.h>
//#include <sys/un.h>
#include <unistd.h>
#endif

#include <iostream>
#include <string>

#include <stdexcept>

#include "StringUtil.h"
#include "MapUtil.h"
#include "ServerCommon.h"
#include "ChannelCommon.h"
#include "ChannelDescriptor.h"
#include "QueueManager.h"
#include "Queue.h"


using namespace OpenSOAP;
using namespace std;

//for LOG
static const std::string CLASS_SIG = "QueueManager";

QueueManager::QueueManager(bool isFwdQueue)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::" << CLASS_SIG << endl;
#endif /* DEBUG */  
  isFwdQueue_ = isFwdQueue;
#if defined(WIN32)
	int
#else
	string
#endif
	  	sockAddr;

  if (isFwdQueue_) {
    sockAddr = FWDQUEUE_SOCKET_ADDR;  
  } 
  else {
    sockAddr = REQQUEUE_SOCKET_ADDR;
  }
  setSocketAddr(sockAddr);

  //CX^X
  if (isFwdQueue_) {
    queuePtr = new Queue(FWDQUEUE_TABLE_FILE, isFwdQueue_);
  }
  else {
    queuePtr = new Queue(REQQUEUE_TABLE_FILE, isFwdQueue_);
  }
}

QueueManager::~QueueManager()
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::~" << CLASS_SIG << endl;
#endif /* DEBUG */  

}

void QueueManager::doProc(int sockfd)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::doProc() arg=[" << sockfd << "]" << endl;
#endif /* DEBUG */

  //f[^M
  std::string readBuf;
  if (0 > read(sockfd, readBuf)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::doProc: read failed" << endl;
    return;
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::doProc: read=["
       << readBuf << "]" << endl;
#endif

  //Mf[^Kvɉĉ͂AԐMʂ𓾂

  MapUtil readDataMap(OpenSOAP::ChnlCommon::ITEM_DELMIT,
		      OpenSOAP::ChnlCommon::KEY_DELMIT);
  readDataMap.insertItem(readBuf);
  
#ifdef DEBUG
  readDataMap.spy();
#endif /* DEBUG */
  
  //bZ[W̉
  std::string response;  
  MapUtil writeDataMap(OpenSOAP::ChnlCommon::ITEM_DELMIT,
		       OpenSOAP::ChnlCommon::KEY_DELMIT);

  if (!readDataMap.exists(OpenSOAP::ChnlCommon::CMD)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::doProc: read failed" << endl;
    return;
  }
  std::string cmd = readDataMap[OpenSOAP::ChnlCommon::CMD];
  
  if (OpenSOAP::ChnlCommon::OPEN == cmd) {
    //P[X
    
    //
    std::string s1(OpenSOAP::ChnlCommon::RESULT);
    std::string s2(OpenSOAP::ChnlCommon::SUCCESS);
    writeDataMap.insert(make_pair(s1,s2));
    //ʐM^Cv
    s1 = OpenSOAP::ChnlCommon::CHNL_TYPE;
    s2 = OpenSOAP::ChnlCommon::UNIX_SOCKET;
    writeDataMap.insert(make_pair(s1,s2));
    
    //ԐMf[^
    response = writeDataMap.toString();
  
    ChannelDescriptor chnlDesc;
    chnlDesc.setReadFd(sockfd);
    chnlDesc.setWriteFd(sockfd);
    
    if (0 > write(sockfd, response)) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::doProc: write failed" << endl;
      return;
    }
    
    //s
    queuePtr->setChnlDesc(chnlDesc);
    queuePtr->execPushOpr();
  }
  else if (OpenSOAP::ChnlCommon::CLOSE == cmd) {
    //\[XJ
#ifdef DEBUG
    cerr << CLASS_SIG << "::doProc: CLOSE proc." << endl;
#endif /* DEBUG */
    
    std::string s1(OpenSOAP::ChnlCommon::RESULT);
    std::string s2(OpenSOAP::ChnlCommon::SUCCESS);
    writeDataMap.insert(make_pair(s1,s2));
    response = writeDataMap.toString();
  
    if (0 > write(sockfd, response)) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::doProc: write failed" << endl;
      return;
    }

  }
#if defined(WIN32)
	closesocket(sockfd);
#else
  close(sockfd);
#endif

#ifdef DEBUG
  cerr << CLASS_SIG << "::doProc: close(" << sockfd << ")" << endl;
#endif /* DEBUG */

}
